==> u61-0.3.2.tar.gz: u61-0.3.2/src/letter.cpp <==
/****************************************************************************
 *                                                                          *
 * U U    6   1            U U   FFF  O   O  TTT                            *
 * U U   6   11   b        U U   F   O O O O  T                             *
 * U U - 66   1   bb  y y  U U - FF  O O O O  T                             *
 * U U   6 6  1   b b  y   U U   F   O O O O  T                             *
 *  U     6   1   bb   y    U    F    O   O   T                             *
 *                                                                          *
 * U61 is another block based game                                          *
 * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
 *                                                                          *
 * This program is free software; you can redistribute it and/or            *
 * modify it under the terms of the GNU General Public License              *
 * as published by the Free Software Foundation; either version 2           *
 * of the License, or (at your option) any later version.                   *
 *                                                                          *
 * This program is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU General Public License for more details.                             *
 *                                                                          *
 * You should have received a copy of the GNU General Public License        *
 * along with this program; if not, write to the Free Software              *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
 *                                                                          *
 * This project is also available on SourceForge  (http://sourceforge.net)  *
 ****************************************************************************/

/*
 * file name:   letter.cpp
 * author:      U-Foot (ufoot@ufoot.org / www.ufoot.org)
 * description: the letter class allows the drawing of funny dancing letters
 *              it contains information about the letter offset and provides
 *              functions to draw it
 */


/*---------------------------------------------------------------------------
 includes
 ---------------------------------------------------------------------------*/

#include "letter.h"
#include "time.h"
#include "utils.h"
#include "square.h"

/*---------------------------------------------------------------------------
 globals
 ---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------
 functions
 ---------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* 
 * creation of a letter
 */ 
U61_Letter::U61_Letter(CL_Font *fnt,char code)
{
  set_font(fnt);
  ascii=code; 
  disable_dance(); 
  disable_blink();
}

/*--------------------------------------------------------------------------*/
/* 
 * frees the letter
 */ 
U61_Letter::~U61_Letter()
{

}

/*--------------------------------------------------------------------------*/
/*
 * enables the dance mode, in which the letters move
 */
void U61_Letter::enable_dance()
{
  if (!dance_state)
    {
      dance_state=true;

      last_time=U61_Time::for_effect();

      change_dx();
      change_dy();
      change_vx();
      change_vy();
    }
}

/*--------------------------------------------------------------------------*/
/*
 * disables the dance mode, in which the letters move
 */
void U61_Letter::disable_dance()
{
  dance_state=false;

  dx=0;
  dy=0;
}

/*--------------------------------------------------------------------------*/
/*
 * Enables the "blinking mode" (used for editing strings)
 */
void U61_Letter::enable_blink()
{
  blink_state=true;
}

/*--------------------------------------------------------------------------*/
/*
 * Disables the "blinking mode" (used for editing strings)
 */
void U61_Letter::disable_blink()
{
  blink_state=false;
}

/*--------------------------------------------------------------------------*/
/*
 * computes all the movements off the letter
 */
void U61_Letter::compute()
{
  int time;

  time=U61_Time::for_effect();
  while(last_time<time)
    {
      dx+=vx;
      dy+=vy;
        
      if (dx<-W_MAX_MOVE||dx>W_MAX_MOVE)
        {
	  dx-=vx;
	  change_vx();
	  //            change_vy();
        }
      if (dy<-H_MAX_MOVE||dy>H_MAX_MOVE)
        {
	  dy-=vy;
	  change_vy();
	  //            change_vx();
        }

      last_time+=SLOW_FACTOR;
    }    
}

/*--------------------------------------------------------------------------*/
/*
 * finds a new vx value
 */
void U61_Letter::change_vx()
{
  if (vx!=1 && vx!=-1)
    {
      vx=1;
    }
  if (U61_Utils::random(2))
    {
      vx=-vx;
    }
}

/*--------------------------------------------------------------------------*/
/*
 * finds a new vy value
 */
void U61_Letter::change_vy()
{
  if (vy!=1 && vy!=-1)
    {
      vy=1;
    }
  if (U61_Utils::random(2))
    {
      vy=-vy;
    }
}

/*--------------------------------------------------------------------------*/
/*
 * finds a new dx value
 */
void U61_Letter::change_dx()
{
  dx=U61_Utils::random(W_MAX_MOVE)-W_MAX_MOVE/2;
}

/*--------------------------------------------------------------------------*/
/*
 * finds a new dy value
 */
void U61_Letter::change_dy()
{
  dy=U61_Utils::random(H_MAX_MOVE)-H_MAX_MOVE/2;
}

/*--------------------------------------------------------------------------*/
/*
 * changes the ascii code of the letter
 */
char U61_Letter::operator =(char code)
{
  ascii=code;

  return code;
}

/*--------------------------------------------------------------------------*/
/*
 * returns the ascii code of a letter
 */
int U61_Letter::get_ascii()
{
  return ascii;
}

/*--------------------------------------------------------------------------*/
/*
 * draws the letter onto a bitmap
 */
void U61_Letter::set_font(CL_Font *fnt)
{
  font=fnt;
}

/*--------------------------------------------------------------------------*/
/*
 * draws the letter onto a bitmap
 */
void U61_Letter::draw(int x,int y)
{
  char text[2];

  if (blink_state && U61_Square::should_blink())
    {
      text[0]='_';
    }
  else
    {
      text[0]=ascii;
    }
  text[1]=0;

  if (dance_state)
    {
      compute();
    }
  font->print_center(x+dx,y+dy,text);
}

/*--------------------------------------------------------------------------*/
/*
 * returns the height of the letter, in pixels
 */
int U61_Letter::get_height()
{
  int height;

  height=font->get_height();
  if (dance_state)
    {
      height+=H_MAX_MOVE;
    }
  return height;
}

/*--------------------------------------------------------------------------*/
/*
 * returns the width of the letter, in pixels
 */
int U61_Letter::get_width()
{
  char text[2];
  int width;

  text[0]=ascii;
  text[1]=0;

  width=font->get_text_width(text);
  if (dance_state)
    {
      width+=W_MAX_MOVE;
    }
  return width;
}

==> u61-0.3.3.tar.gz: u61-0.3.3/src/letter.cpp <==
/****************************************************************************
 *                                                                          *
 * U U    6   1            U U   FFF  O   O  TTT                            *
 * U U   6   11   b        U U   F   O O O O  T                             *
 * U U - 66   1   bb  y y  U U - FF  O O O O  T                             *
 * U U   6 6  1   b b  y   U U   F   O O O O  T                             *
 *  U     6   1   bb   y    U    F    O   O   T                             *
 *                                                                          *
 * U61 is another block based game                                          *
 * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
 *                                                                          *
 * This program is free software; you can redistribute it and/or            *
 * modify it under the terms of the GNU General Public License              *
 * as published by the Free Software Foundation; either version 2           *
 * of the License, or (at your option) any later version.                   *
 *                                                                          *
 * This program is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU General Public License for more details.                             *
 *                                                                          *
 * You should have received a copy of the GNU General Public License        *
 * along with this program; if not, write to the Free Software              *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
 *                                                                          *
 * This project is also available on SourceForge  (http://sourceforge.net)  *
 ****************************************************************************/

/*
 * file name:   letter.cpp
 * author:      U-Foot (ufoot@ufoot.org / www.ufoot.org)
 * description: the letter class allows the drawing of funny dancing letters
 *              it contains information about the letter offset and provides
 *              functions to draw it
 */


/*---------------------------------------------------------------------------
 includes
 ---------------------------------------------------------------------------*/

#include "letter.h"
#include "time.h"
#include "utils.h"
#include "square.h"

/*---------------------------------------------------------------------------
 globals
 ---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------
 functions
 ---------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* 
 * creation of a letter
 */ 
U61_Letter::U61_Letter(CL_Font *fnt,char code)
{
  set_font(fnt);
  ascii=code; 
  disable_dance(); 
  disable_blink();
}

/*--------------------------------------------------------------------------*/
/* 
 * frees the letter
 */ 
U61_Letter::~U61_Letter()
{

}

/*--------------------------------------------------------------------------*/
/*
 * enables the dance mode, in which the letters move
 */
void U61_Letter::enable_dance()
{
  if (!dance_state)
    {
      dance_state=true;

      last_time=U61_Time::for_effect();

      change_dx();
      change_dy();
      change_vx();
      change_vy();
    }
}

/*--------------------------------------------------------------------------*/
/*
 * disables the dance mode, in which the letters move
 */
void U61_Letter::disable_dance()
{
  dance_state=false;

  dx=0;
  dy=0;
}

/*--------------------------------------------------------------------------*/
/*
 * Enables the "blinking mode" (used for editing strings)
 */
void U61_Letter::enable_blink()
{
  blink_state=true;
}

/*--------------------------------------------------------------------------*/
/*
 * Disables the "blinking mode" (used for editing strings)
 */
void U61_Letter::disable_blink()
{
  blink_state=false;
}

/*--------------------------------------------------------------------------*/
/*
 * computes all the movements off the letter
 */
void U61_Letter::compute()
{
  int time;

  time=U61_Time::for_effect();
  while(last_time<time)
    {
      dx+=vx;
      dy+=vy;
        
      if (dx<-W_MAX_MOVE||dx>W_MAX_MOVE)
        {
	  dx-=vx;
	  change_vx();
	  //            change_vy();
        }
      if (dy<-H_MAX_MOVE||dy>H_MAX_MOVE)
        {
	  dy-=vy;
	  change_vy();
	  //            change_vx();
        }

      last_time+=SLOW_FACTOR;
    }    
}

/*--------------------------------------------------------------------------*/
/*
 * finds a new vx value
 */
void U61_Letter::change_vx()
{
  if (vx!=1 && vx!=-1)
    {
      vx=1;
    }
  if (U61_Utils::random(2))
    {
      vx=-vx;
    }
}

/*--------------------------------------------------------------------------*/
/*
 * finds a new vy value
 */
void U61_Letter::change_vy()
{
  if (vy!=1 && vy!=-1)
    {
      vy=1;
    }
  if (U61_Utils::random(2))
    {
      vy=-vy;
    }
}

/*--------------------------------------------------------------------------*/
/*
 * finds a new dx value
 */
void U61_Letter::change_dx()
{
  dx=U61_Utils::random(W_MAX_MOVE)-W_MAX_MOVE/2;
}

/*--------------------------------------------------------------------------*/
/*
 * finds a new dy value
 */
void U61_Letter::change_dy()
{
  dy=U61_Utils::random(H_MAX_MOVE)-H_MAX_MOVE/2;
}

/*--------------------------------------------------------------------------*/
/*
 * changes the ascii code of the letter
 */
char U61_Letter::operator =(char code)
{
  ascii=code;

  return code;
}

/*--------------------------------------------------------------------------*/
/*
 * returns the ascii code of a letter
 */
int U61_Letter::get_ascii()
{
  return ascii;
}

/*--------------------------------------------------------------------------*/
/*
 * draws the letter onto a bitmap
 */
void U61_Letter::set_font(CL_Font *fnt)
{
  font=fnt;
}

/*--------------------------------------------------------------------------*/
/*
 * draws the letter onto a bitmap
 */
void U61_Letter::draw(int x,int y)
{
  char text[2];

  if (blink_state && U61_Square::should_blink())
    {
      text[0]='_';
    }
  else
    {
      text[0]=ascii;
    }
  text[1]=0;

  if (dance_state)
    {
      compute();
    }
  font->print_center(x+dx,y+dy-font->get_height()/2,text);
}

/*--------------------------------------------------------------------------*/
/*
 * returns the height of the letter, in pixels
 */
int U61_Letter::get_height()
{
  int height;

  height=font->get_height();
  if (dance_state)
    {
      height+=H_MAX_MOVE;
    }
  return height;
}

/*--------------------------------------------------------------------------*/
/*
 * returns the width of the letter, in pixels
 */
int U61_Letter::get_width()
{
  char text[2];
  int width;

  text[0]=ascii;
  text[1]=0;

  width=font->get_text_width(text);
  if (dance_state)
    {
      width+=W_MAX_MOVE;
    }
  return width;
}
